#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef int (*compare_fun)(void *a, void *b);

struct parray {
        size_t Count;
        size_t SIZE;
        void **DATA;
        compare_fun F_compare;
};

int
compare(void *x1, void *x2)
{
        if(x1<x2) return -1;
        if(x1>x2) return  1;
        return  0;
}

/* Create new pointers array. */
struct parray *
parray_new(compare_fun F_compare)
{
        struct parray *V = malloc(sizeof(*V));
        if(V == NULL){
                fprintf(stderr, "can't allocate parray!\n");
                return NULL;
        }
        V->DATA = malloc(sizeof(void *) * 10);
        if(V->DATA == NULL){
                fprintf(stderr, "can't allocate parray!\n");
                free(V);
                return NULL;
        }
        V->Count = 0;
        V->SIZE  = 10;
        V->F_compare = F_compare;
        return V;
};

int
parray_count(struct parray * V){return V->Count;}

/* Add pointer to array. */
int
parray_add(struct parray * V, void *p)
{
        if(V->Count == V->SIZE){
                void *p;
                p = realloc(V->DATA, V->SIZE*sizeof(void *)*2);
	        if (p == NULL){
	                fprintf(stderr, "can't realloc parray data\n");
	                return -1;
	        }
	        V->SIZE *= 2;
                V->DATA = p;
        }
        V->DATA[V->Count] = p;
        V->Count++;
        return V->Count;
}

/* Get pointer at position pos. */
void*
parray_get_nth(struct parray * V, size_t Index)
{
        if(Index>=V->Count){
                fprintf(stderr, "cann't get pointer!\n");
                return NULL;
        }
        else {
                return V->DATA[Index];
        }
}

/* Change pointer at position nth. */
void
parray_change(struct parray * V, size_t Index, void *new_p)
{
        if(Index>=V->Count){
                fprintf(stderr, "element cann't change!\n");
                return;
        }
        else {
                V->DATA[Index] = new_p;
        }
}

/* Search sorted array of pointers using compare function compare_func. */
int
parray_search(struct parray *V, void *p)
{
        size_t i;
        for(i=0; i<V->Count; i++)
        if ((V->F_compare)(p, V->DATA[i]) == 0)
                return i;
        return -1;
}

/* Sort array of pointers using compare function compare_func. */
void
parray_sort(struct parray *V)
{
        size_t i,j;
        void *p;
        for(i=0;i<V->Count-1;i++)
                for(j=i+1;j<V->Count;j++)
                        if((V->F_compare)(V->DATA[i], V->DATA[j]) == 1){
                                p = V->DATA[i];
                                V->DATA[i] = V->DATA[j];
                                V->DATA[j] = p;
                        }
}

/* Delete pointer from array at position nth. */
void
parray_delete_nth(struct parray * V, size_t Index)
{
        if(Index>=V->Count){
                fprintf(stderr, "element cann't delete!\n");
                return;
        }
        else {
                memmove(&V->DATA[Index], &V->DATA[Index+1], (V->Count-Index)*sizeof(void *));
                V->Count--;
        }
}

/* Delete range of pointers from array starting from nth up to count element. */
void
parray_delete_range(struct parray *V, unsigned int n, size_t Count)
{
        if (n >= V->Count){
                fprintf(stderr, "cann't delete range!\n");
		return;
	}
        else {
	        if(Count >= V->Count - n){
                        V->Count = n;
	        }
                else {
                        memmove(V->DATA+n, V->DATA+n+Count, sizeof(void *)*(V->Count-Count-n));
                        V->Count = V->Count - Count;
	        }
        }
}

struct user_data{
	int age;
};

int
print_age(void *a, void *b)
{
	struct user_data *ud = (struct user_data *) b;
	printf("%c, age: %d\n", a, ud->age);
	return 0;
}

int
callback(void *a, void *b)
{
	struct user_data *ud = (struct user_data *) b;
	printf("%f - %d\n", (int*)a, ud->age);
        return 0;
}

/* Traverse pointer array calling callback function for each element. */
void
parray_foreach(struct parray *V, compare_fun fun, void *b)
{
        int i;
	for (i = 0; i <= V->Count-1; i++)
                if (fun(V->DATA[i], b) != 0) 
                        return;
}

int main()
{
        struct user_data ud;
        struct parray *V = parray_new(compare);

        parray_add(V, (char *)'R');
        parray_add(V, (char *)'A');
        parray_add(V, (char *)'M');
        parray_add(V, (char *)'B');
        parray_add(V, (char *)'O');

        printf("%c%c%c%c%c\n", parray_get_nth(V, 0), parray_get_nth(V, 1),
        parray_get_nth(V, 2), parray_get_nth(V, 3), parray_get_nth(V, 4));

        parray_delete_nth(V, 1);

        printf("%c%c%c%c\n", parray_get_nth(V, 0), parray_get_nth(V, 1),
        parray_get_nth(V, 2), parray_get_nth(V, 3));

        parray_change(V, 3, (void *)'!');
        parray_add(V, (char *)'O');

        printf("%c%c%c%c%c\n", parray_get_nth(V, 0), parray_get_nth(V, 1),
        parray_get_nth(V, 2), parray_get_nth(V, 3), parray_get_nth(V, 4));

        printf("%d\n", parray_search(V, (void *)'M'));

        parray_add(V, (char *)'1');
        parray_sort(V);

        printf("%c%c%c%c%c%c\n", parray_get_nth(V, 0), parray_get_nth(V, 1),
        parray_get_nth(V, 2), parray_get_nth(V, 3), parray_get_nth(V, 4),
        parray_get_nth(V, 5));

        printf("%d\n", V->Count);

        ud.age = 25;
        parray_foreach(V, print_age, &ud);

        getchar();
        return 0;
}
//---------------------------------------------------------------------------
